import robloxpy.User.Internal as Internal
import robloxpy.Utils as Utils
import robloxpy.Game.External as External

#Personal Game Vars
class MyGame(object):
    def __init__(self):
        self.maxPlayerCount
        self.socialSlotType
        self.customSocialSlotsCount
        self.allowCopying
        self.currentSavedVersion
        self.name
        self.isRootPlace
        self.descriptionisRootPlace

def GetUniverseID(PlaceID):
    try:
        response = Internal.CurrentCookie.get(f"{Utils.GamesAPI}games/multiget-place-details?placeIds={str(PlaceID)}")
        try:
            return response.json()['data'][0]
        except:
            return 'Universe Not Found'
    except Exception as e:
        return e

def GetCurrentPlayers(PlaceID):
    try:
        UniverseID = GetUniverseID(PlaceID)
        GameData = External.GetCurrentUniversePlayers(UniverseID)
        return GameData['playing']
    except:
        return "Error"

def GetGameVisits(PlaceID):
    try:
        UniverseID = GetUniverseID(PlaceID)
        GameData = External.GetUniverseData(UniverseID)
        return GameData['visits']
    except:
        return "Error"

def GetGameLikes(PlaceID):
    try:
        UniverseID = GetUniverseID(PlaceID)
        return External.GetUniverseVotes(UniverseID)['upVotes']
    except:
        return "Error"

def GetGameDislikes(PlaceID):
    try:
        UniverseID = GetUniverseID(PlaceID)
        return External.GetUniverseVotes(UniverseID)['downVotes']
    except:
        return "Error"

def GetGameFavourites(PlaceID):
    try:
        UniverseID = GetUniverseID(PlaceID)
        return External.GetUniverseFavourites(UniverseID)
    except:
        return "Error"

def GetMyGameData(PlaceID):
    try:
        response = Internal.CurrentCookie.get(Utils.DevelopAPIV2 + f"places/{PlaceID}")
        print((Utils.DevelopAPIV2 + f"places/{PlaceID}"))
        MyGame.maxPlayerCount = response.json()['maxPlayerCount']
        MyGame.socialSlotType = response.json()['socialSlotType']
        MyGame.customSocialSlotsCount = response.json()['customSocialSlotsCount']
        MyGame.allowCopying = response.json()['allowCopying']
        MyGame.currentSavedVersion = response.json()['currentSavedVersion']
        MyGame.name = response.json()['name']
        MyGame.descriptionisRootPlace = response.json()['description']
        MyGame.isRootPlace = response.json()['isRootPlace']
        return "Saves"
    except:
        return response.json()['errors'][0]['message']


import requests,json
from robloxpy import Utils as Utils

def GetGroups(UserID):
    """
    Returns the list of groups a user is in
    [Name],[ID]
    """
    response = requests.get(Utils.APIURL + f"users/{UserID}/groups")
    FullList = []
    IDList = []
    Grouplist = json.loads(response.text)
    for group in Grouplist:
        FullList.append(group['Name'])
        IDList.append(group['Id'])
    return FullList, IDList


import requests,json
import robloxpy.User.Internal as Internal
import robloxpy.Utils as Utils

def BuyItem(MarketID):
    Data = requests.get(f"https://api.roblox.com/marketplace/productinfo?assetId={MarketID}").json()
    sellerId = Data['Creator']['Id']
    Price = Data['PriceInRobux']
    if(Price == None):
        Price = 0
    productId = Data['ProductId']
    response = Internal.CurrentCookie.post(f"https://economy.roblox.com/v1/purchases/products/{productId}",data={"expectedCurrency":1,"expectedPrice":Price,"expectedSellerId":sellerId})
    return response.json()['reason']


import requests,json
from robloxpy import Utils as Utils


global Owner
global Name
global isOwned
global Description
global Shout
global Roles
global isPublic
global isBc


def IsGroupOwned(GroupID):
    """
    Returns whether a group is owned
    """
    response = requests.get(Utils.GroupAPIV1 + str(GroupID))
    try:
        if(str(response.json()['Owner']) == 'None'):
            return False
        else:
            return True
    except:
        return response.json()['errors'][0]['message']

def GetName(GroupID):
    """
    Returns the name of a group
    """
    response = requests.get(Utils.APIURL + f"groups/{GroupID}")
    try:
        return response.json()['Name']
    except:
        return response.json()['errors'][0]['message']

def GetOwner(GroupID):
    """
    Returns the name of the owner of a group
    """
    response = requests.get(Utils.APIURL + f"groups/{GroupID}")
    try:
        return response.json()['Owner']
    except:
        return response.json()['errors'][0]['message']

def GetDescription(GroupID):
    """
    Returns the description of a group
    """
    response = requests.get(Utils.APIURL + f"groups/{GroupID}")
    try:
        return response.json()['Description']
    except:
        return response.json()['errors'][0]['message']

def GetEmblem(GroupID):
    """
    Returns the URL of the group emblem
    """
    response = requests.get(Utils.APIURL + f"groups/{GroupID}")
    try:
        return response.json()['EmblemUrl']
    except:
        return response.json()['errors'][0]['message']

def GetRoles(GroupID):
    """
    Returns the roles and ranks of a group

    [Role Names],[Role Ranks]
    """
    response = requests.get(Utils.APIURL + f"groups/{GroupID}")
    try:
        FullList = []
        RankList = []
        GroupRanks = response.json()['Roles']
        for Rank in GroupRanks:
            FullList.append((Rank['Name']))
            RankList.append((Rank['Rank']))
        return FullList,RankList
    except:
        return response.json()

def GetAllies(GroupID):
    """
    Returns a list of groups that are allies
    """
    response = requests.get(Utils.APIURL + f"groups/{GroupID}/allies")
    try:
        FullList = []
        Grouplist = json.loads(response.text)
        Grouplist = Grouplist['Groups']
        for group in Grouplist:
            FullList.append(group['Name'])
        return FullList
    except:
        return response.json()['errors'][0]['message']

def GetEnemies(GroupID):
    """
    Returns a list of groups that are enemies
    """
    response = requests.get(Utils.APIURL + f"groups/{GroupID}/enemies")
    try:
        FullList = []
        Grouplist = json.loads(response.text)
        Grouplist = Grouplist['Groups']
        for group in Grouplist:
            FullList.append(group['Name'])
        return FullList
    except:
        return response.json()['errors'][0]['message']

def GetMemberCount(GroupID):
    """
    Returns a count of how many users are in a group
    """
    response = requests.get(Utils.GroupAPIV1 + f"{GroupID}")
    try:
        return response.json()['memberCount']
    except:
        return response.json()['errors'][0]['message']

def isPublic(GroupID):
    """
    Returns if a group is public to join
    """
    response = requests.get(Utils.GroupAPIV1 + f"groups/{GroupID}")
    try:
        return response.json()['publicEntryAllowed']
    except:
        return response.json()['errors'][0]['message']

def isBCOnly(GroupID):
    """
    Returns if a group is Builderclub Only
    """
    response = requests.get(Utils.GroupAPIV1 + f"groups/{GroupID}")
    try:
        return response.json()['isBuildersClubOnly']
    except:
        return response.json()['errors'][0]['message']

def GetMembersList(GroupID, Limit = (999999999999)):
    """
    Returns a full list of group members

    [Username],[ID]
    """
    MemberList = []
    IDList = []
    CurrentAmount = 0
    Cursor = 'None'
    response = requests.get(Utils.GroupAPIV1 + (f"groups/{GroupID}/users?SortOrder=Asc&limit=100"))
    try:
        while (Cursor != 'null'):
            if(Cursor is None):
                Cursor = 'null'
            else:
                try:
                    Cursor = response.json()['nextPageCursor']
                except:
                    Cursor = 'null'
            for Member in response.json()['data']:
                if(CurrentAmount < Limit):
                    MemberList.append(Member['user']['username'])
                    IDList.append(Member['user']['userId'])
                    CurrentAmount +=1
                else:
                    return MemberList,IDList
            if(Cursor is None):
                Cursor = 'null'
            else:
                try:
                    response = requests.get(Utils.GroupAPIV1 + (f"groups/{GroupID}/users?SortOrder=Asc&limit=100&cursor={Cursor}"))
                except:
                    Cursor = 'null'
        return MemberList, IDList
    except:
        return response.json()['errors'][0]['message']

def GetMembersinRoleList(GroupID,RoleID,Limit= (999999999999)):
    """
    Returns a list of users in a specific role
    [Username],[ID]
    """
    MemberList = []
    IDList = []
    CurrentAmount = 0
    Cursor = 'None'
    response = requests.get(Utils.GroupAPIV1 + (f"groups/{GroupID}/roles/{RoleID}/users?SortOrder=Asc&limit=100"))
    #return response.json()
    try:
        while (Cursor != 'null'):
            if(Cursor is None):
                Cursor = 'null'
            else:
                try:
                    Cursor = response.json()['nextPageCursor']
                except:
                    Cursor = 'null'
            for Member in response.json()['data']:
                if(CurrentAmount < Limit):
                    MemberList.append(Member['username'])
                    IDList.append(Member['userId'])
                    CurrentAmount +=1
                else:
                    return MemberList,IDList
            if(Cursor is None):
                Cursor = 'null'
            else:
                try:
                    response = requests.get(Utils.GroupAPIV1 + (f"groups/{GroupID}/roles/{RoleID}/users?SortOrder=Asc&limit=100&cursor={Cursor}"))
                except:
                    Cursor = 'null'
        return MemberList, IDList
    except:
        return response.json()['errors'][0]['message']



import requests,os
import robloxpy.User.Internal

TestCookie = ""


#URL List

MobileAPI = "https://www.roblox.com/mobileapi/"
FriendsAPI = "https://friends.roblox.com/v1/users/"
APIURL = "https://api.roblox.com/"
UserAPI = "https://api.roblox.com/users/"
UserAPIV1 = "https://users.roblox.com/v1/users/"
GroupAPIV1 = "https://groups.roblox.com/v1/groups/"
GroupAPIV2 = "https://groups.roblox.com/v2/"
EconomyURL = "https://economy.roblox.com/v1/"
EconomyURLV2 = "https://economy.roblox.com/v2/"
InventoryURL = "https://inventory.roblox.com/v2/assets/"
Inventory1URL = "https://inventory.roblox.com/v1/users/"
SettingsURL = "https://www.roblox.com/my/settings/json"
PrivateMessageAPIV1 = "https://privatemessages.roblox.com/v1/"
GamesAPI = "https://games.roblox.com/v1/"
DevelopAPIV2 = "https://develop.roblox.com/v2/"
GameAuthUrl = "https://auth.roblox.com/v1/authentication-ticket/"



Version = "0.2.14"
WIP = "Not Implemented Yet"

def CheckForUpdate():
    """
    Returns whether a new version of robloxpy is availible.
    """
    response = requests.get('https://pypi.org/pypi/robloxpy/json')
    LatestVersion = response.json()['info']['version']
    if(Version == LatestVersion):
        return 'You are up to date!'
    else:
        return f'Version {LatestVersion} is now availible'

def GetVersion():
    """
    Returns the current version of robloxpy that is being used.
    """
    return Version

def UpdateInstructions(Version = "Latest"):
    """
    Provides Instructions to update robloxpy
    """
    Version = str(Version)
    if(Version == "Latest"):
        return "Update robloxpy through pip using following command: \n'pip install robloxpy --upgrade'"
    else:
        return f"This software is intended to work on robloxpy version {Version} please install using the following command \n'pip install robloxpy=={Version}'\n If you get an error with this command the developer of this tool has not provided a valid version"

def SetProxy(ProxyIP):
    """
    Set the proxy to currently be used, this is global
    Format: IP:Port
    """
    #Format 144.217.101.245:3129
    if(ProxyIP != None):
        proxy = 'http://' + str(ProxyIP)
        os.environ['http_proxy'] = proxy
        os.environ['https_proxy'] = proxy

def CheckProxy(proxyAddress=None):
    """
    Check the current IP address being given by the program
    """
    SetProxy(proxyAddress)
    response = requests.get('https://api.ipify.org/?format=json')
    return response.json()['ip']

def CheckCookie(Cookie = None):
    """
    If you want to check the current used cookie just run the function without any variable, if you wish to check a specific cookie then enter the cookie as a string
    """
    try:
        if(Cookie == None):
            session = robloxpy.User.Internal.CurrentCookie
        else:
            session = requests.session()
            CurrentCookie = {'.ROBLOSECURITY': Cookie}
            requests.utils.add_dict_to_cookiejar(session.cookies, CurrentCookie)
            Header = session.post('https://catalog.roblox.com/')
            session.headers['X-CSRF-TOKEN'] = Header.headers['X-CSRF-TOKEN']
        response = session.get(MobileAPI + 'userinfo')
        try:
            Temp = response.json()['UserID']
            #return response.json()
            return "Valid Cookie"
        except:
            return "Invalid Cookie"
    except:
        return "No Cookie Set"



import requests,json,random,webbrowser
from time import time
from robloxpy import Utils as Utils

RawCookie = None
UserID = None
Username = None
Robux = None
Thumbnail = None
isBuildersclub = None
isPremium = None
canChangeUsername = None
isAdmin = None
isEmailOnFile = None
isEmailVerified = None
isPhoneFeatureEnabled = None
isSuperSafePrivacyMode = None
IsAppChatSettingEnabled = None
IsGameChatSettingEnabled = None
IsContentRatingsSettingEnabled = None
IsParentalControlsTabEnabled = None
IsSetPasswordNotificationEnabled = None
ChangePasswordRequiresTwoStepVerification = None
ChangeEmailRequiresTwoStepVerification = None
UserEmail = None
UserEmailMasked = None
UserEmailVerified = None
CanHideInventory = None
CanTrade = None
MissingParentEmail = None
IsUpdateEmailSectionShown = None
IsUnder13UpdateEmailMessageSectionShown = None
IsUserConnectedToFacebook = None
IsTwoStepToggleEnabled = None
AgeBracket = None
UserAbove13 = None
ClientIpAddress = None
UserAge = None
IsBcRenewalMembership = None
IsAccountPinEnabled = None
IsAccountRestrictionsFeatureEnabled = None
IsAccountRestrictionsSettingEnabled = None
IsAccountSettingsSocialNetworksV2Enabled = None
InApp = None
HasFreeNameChange = None
IsAgeDownEnabled = None
ReceiveNewsletter = None


def SetCookie(Cookie,Details = True):
    """
    Set the current cookie for internal commands, if you wish to only perform an action set 'Details' to 'False' to prevent un-needed requests being made.
    """
    try:
        global RawCookie
        global CurrentCookie
        RawCookie = Cookie
        session = requests.session()
        CurrentCookie = {'.ROBLOSECURITY': Cookie}
        requests.utils.add_dict_to_cookiejar(session.cookies, CurrentCookie)
        Header = session.post('https://catalog.roblox.com/')
        session.headers['X-CSRF-TOKEN'] = Header.headers['X-CSRF-TOKEN']
        session.headers["Origin"] = "https://www.roblox.com"
        session.headers["Referer"] = "https://www.roblox.com/"
        CurrentCookie = session
        if(Utils.CheckCookie(CurrentCookie) == "Invalid"):
            return "Invalid Cookie"
        if(Details == True):
            GetDetails()
        else:
            GetDetails(False)
        return "Cookie Set"
    except:
        return "Error Setting Cookie"

def GetDetails(Details = True):
    """
    Internal function to get the details of the current cookie, can be used if you have statements to determine if you want the details or not
    """
    global Gamesession
    global UserID
    global Username
    global Robux
    global Thumbnail
    global isBuildersclub
    global isPremium
    global canChangeUsername
    global isAdmin
    global isEmailOnFile
    global isEmailVerified
    global isPhoneFeatureEnabled
    global isSuperSafePrivacyMode
    global IsAppChatSettingEnabled
    global IsGameChatSettingEnabled
    global IsContentRatingsSettingEnabled
    global IsParentalControlsTabEnabled
    global IsSetPasswordNotificationEnabled
    global ChangePasswordRequiresTwoStepVerification
    global ChangeEmailRequiresTwoStepVerification
    global UserEmail
    global UserEmailMasked
    global UserEmailVerified
    global CanHideInventory
    global CanTrade
    global MissingParentEmail
    global IsUpdateEmailSectionShown
    global IsUnder13UpdateEmailMessageSectionShown
    global IsUserConnectedToFacebook
    global IsTwoStepToggleEnabled
    global AgeBracket
    global UserAbove13
    global ClientIpAddress #Shows the IP address roblox sees
    global UserAge
    global IsBcRenewalMembership
    global IsAccountPinEnabled
    global IsAccountRestrictionsFeatureEnabled
    global IsAccountRestrictionsSettingEnabled
    global IsAccountSettingsSocialNetworksV2Enabled
    global InApp
    global HasFreeNameChange
    global IsAgeDownEnabled
    global ReceiveNewsletter
    if(Details == True):
        try:
            SettingsContainer = CurrentCookie.get(Utils.SettingsURL)
            SettingsContainer = SettingsContainer.json()

            BasicContainer = CurrentCookie.get(Utils.MobileAPI + 'userinfo')
            BasicContainer = BasicContainer.json()

            UserID = BasicContainer['UserID']
            Username = BasicContainer['UserName']
            Robux = BasicContainer['RobuxBalance']
            Thumbnail = BasicContainer['ThumbnailUrl']
            isBuildersclub = BasicContainer['IsAnyBuildersClubMember']
            isPremium = BasicContainer['IsPremium']
            canChangeUsername = SettingsContainer['ChangeUsernameEnabled']
            isAdmin = SettingsContainer['IsAdmin']
            isEmailOnFile = SettingsContainer['IsEmailOnFile']
            isEmailVerified = SettingsContainer['IsEmailVerified']
            isPhoneFeatureEnabled = SettingsContainer['IsPhoneFeatureEnabled']
            isSuperSafePrivacyMode = SettingsContainer['UseSuperSafePrivacyMode']
            IsAppChatSettingEnabled = SettingsContainer['IsAppChatSettingEnabled']
            isGameChatSettingEnabled = SettingsContainer['IsGameChatSettingEnabled']
            isContentRatingsSettingEnabled = SettingsContainer['IsContentRatingsSettingEnabled']
            isParentalControlsTabEnabled = SettingsContainer['IsParentalControlsTabEnabled']
            isSetPasswordNotificationEnabled = SettingsContainer['IsSetPasswordNotificationEnabled']
            ChangePasswordRequiresTwoStepVerification = SettingsContainer['ChangePasswordRequiresTwoStepVerification']
            ChangeEmailRequiresTwoStepVerification = SettingsContainer['ChangeEmailRequiresTwoStepVerification']
            UserEmail = SettingsContainer['UserEmail']
            UserEmailMasked = SettingsContainer['UserEmailMasked']
            UserEmailVerified = SettingsContainer['UserEmailVerified']
            CanHideInventory = SettingsContainer['CanHideInventory']
            CanTrade = SettingsContainer['CanTrade']
            MissingParentEmail = SettingsContainer['MissingParentEmail']
            isUnder13UpdateEmailMessageSectionShown = SettingsContainer['IsUnder13UpdateEmailMessageSectionShown']
            isUserConnectedToFacebook = SettingsContainer['IsUserConnectedToFacebook']
            isUpdateEmailSectionShown = SettingsContainer['IsUpdateEmailSectionShown']
            isTwoStepToggleEnabled = SettingsContainer['IsTwoStepToggleEnabled']
            AgeBracket = SettingsContainer['AgeBracket']
            UserAbove13 = SettingsContainer['UserAbove13']
            ClientIpAddress = SettingsContainer['ClientIpAddress']
            UserAge = SettingsContainer['AccountAgeInDays']
            isBcRenewalMembership = SettingsContainer['IsBcRenewalMembership']
            isAccountPinEnabled = SettingsContainer['IsAccountPinEnabled']
            isAccountRestrictionsFeatureEnabled = SettingsContainer['IsAccountRestrictionsFeatureEnabled']
            isAccountRestrictionsSettingEnabled = SettingsContainer['IsAccountRestrictionsSettingEnabled']
            isAccountSettingsSocialNetworksV2Enabled = SettingsContainer['IsAccountSettingsSocialNetworksV2Enabled']
            InApp = SettingsContainer['InApp']
            HasFreeNameChange = SettingsContainer['HasFreeNameChange']
            isAgeDownEnabled = SettingsContainer['IsAgeDownEnabled']
            ReceiveNewsletter = SettingsContainer['ReceiveNewsletter']

            Gamesession = requests.session()
            Gamesession.cookies[".ROBLOSECURITY"] = RawCookie

            Gamesession = Gamesession.post(
            url = Utils.GameAuthUrl,
            headers = {
                "Referer": "https://www.roblox.com/",
                "X-CSRF-Token": Gamesession.post(
                    url = Utils.GameAuthUrl
                ).headers["X-CSRF-Token"],
            }
            ).headers["RBX-Authentication-Ticket"]

            return "Data Gathered"
        except Exception as e:
            return f"Error {e}"
    else:
        UserID = None
        Username = None
        Robux = None
        Thumbnail = None
        isBuildersclub = None
        isPremium = None
        canChangeUsername = None
        isAdmin = None
        isEmailOnFile = None
        isEmailVerified = None
        isPhoneFeatureEnabled = None
        isSuperSafePrivacyMode = None
        IsAppChatSettingEnabled = None
        IsGameChatSettingEnabled = None
        IsContentRatingsSettingEnabled = None
        IsParentalControlsTabEnabled = None
        IsSetPasswordNotificationEnabled = None
        ChangePasswordRequiresTwoStepVerification = None
        ChangeEmailRequiresTwoStepVerification = None
        UserEmail = None
        UserEmailMasked = None
        UserEmailVerified = None
        CanHideInventory = None
        CanTrade = None
        MissingParentEmail = None
        IsUpdateEmailSectionShown = None
        IsUnder13UpdateEmailMessageSectionShown = None
        IsUserConnectedToFacebook = None
        IsTwoStepToggleEnabled = None
        AgeBracket = None
        UserAbove13 = None
        ClientIpAddress #Shows the IP address roblox sees = None
        UserAge = None
        IsBcRenewalMembership = None
        IsAccountPinEnabled = None
        IsAccountRestrictionsFeatureEnabled = None
        IsAccountRestrictionsSettingEnabled = None
        IsAccountSettingsSocialNetworksV2Enabled = None
        InApp = None
        HasFreeNameChange = None
        IsAgeDownEnabled = None
        ReceiveNewsletter = None
        return "Data Not Wanted"

def isFollowing(targetUserID):
    """
    Checks if the current account is following a user
    """
    try:
        response = CurrentCookie.get(f"{Utils.APIURL}user/following-exists?UserID={str(targetUserID)}&followerUserID={UserID}", data={'targetUserID': targetUserID})
        return response.json()['isFollowing']
    except Exception as e:
        return e

def FollowUser(targetUserID):
    """
    Follows a user
    """
    try:
        response = CurrentCookie.post(f"{Utils.FriendsAPI}{str(targetUserID)}/follow", data={'targetUserID': targetUserID})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']
    except Exception as e:
        return e

def UnfollowUser(targetUserID):
    """
    unfollows a user
    """
    try:
        response = CurrentCookie.post(f"{Utils.FriendsAPI}{str(targetUserID)}/unfollow", data={'targetUserID': targetUserID})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']
    except Exception as e:
        return response.json()

def BlockUser(targetUserID):
    """
    Blocks a user
    """
    try:
        response = CurrentCookie.post(f"{Utils.APIURL}userblock/block?userId={str(targetUserID)}", data={'targetUserID': targetUserID})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']
    except Exception as e:
        return response.json()

def GetBlockedUsers():
    """
    Returns users which are blocked

    [UserID],[UserName]
    """
    try:
        response = CurrentCookie.get(f"{Utils.SettingsURL}")
        Data = response.json()['BlockedUsersModel']['BlockedUsers']
        BlockedIDs = []
        BlockedNames = []

        for User in Data:
            BlockedIDs.append(User['uid'])
            BlockedNames.append(User['Name'])
        return BlockedIDs, BlockedNames
    except:
        return response.json()

def UnblockUser(targetUserID):
    """
    unblocks a user
    """
    try:
        #response = CurrentCookie.post(f"{Utils.APIURL}userblock/unblock?userId={str(targetUserID)}", data={'targetUserID': targetUserID})
        response = CurrentCookie.post(f"https://accountsettings.roblox.com/v1/users/{targetUserID}/unblock", data={'targetUserID': targetUserID})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']
    except Exception as e:
        return response.json()

def SendMessage(targetUserID,Subject,Body):
    response = None
    try:
        response = CurrentCookie.post(Utils.PrivateMessageAPIV1 + 'messages/send/', data={
                            'userId': UserID,
                            'subject': Subject,
                            'body': Body,
                            'recipientid': targetUserID,
                            })
        return response.json()['message']
    except Exception as e:
        return response.json()

def JoinGame(PlaceId):
    BrowserID = random.randint(10000000000, 99999999999)
    webbrowser.open(f"roblox-player:1+launchmode:play+gameinfo:{Gamesession}+launchtime:{int(time()*1000)}+placelauncherurl:https%3A%2F%2Fassetgame.roblox.com%2Fgame%2FPlaceLauncher.ashx%3Frequest%3DRequestGame%26browserTrackerId%3D{BrowserID}%26placeId%3D{PlaceId}%26isPlayTogetherGame%3Dfalse+browsertrackerid:{BrowserID}+robloxLocale:en_us+gameLocale:en_us")




















import requests,json,datetime
from robloxpy import Utils as Utils
def GetID(Username):
    """
    Returns the ID of a user based on the username
    """
    response = requests.get(
        Utils.APIURL + f'users/get-by-username?username={Username}')
    try:
        return response.json()['Id']
    except:
        return response.json()['errorMessage']

def GetUserName(UserID):
    """
    Returns the username of a user based on the ID
    """
    response = requests.get(Utils.UserAPI + f"{str(UserID)}")
    try:
        return response.json()['Username']
    except:
        return "Unable to convert ID"

def UsernameHistory(UserID):
    """
    Returns an array of previous usernames as user has had
    """
    Cursor = ""
    Done = False
    PastNames = []
    while(Done == False):
        response = requests.get(Utils.UserAPIV1 + f"{UserID}/username-history?limit=100&sortOrder=Asc&cursor={Cursor}")
        Names = response.json()['data']
        if((response.json()['nextPageCursor'] == "null") or response.json()['nextPageCursor'] == None):
            Done = True
        else:
            Done = False
            Cursor = response.json()['nextPageCursor']
        for Name in Names:
            PastNames.append(Name['name'])
        if(response.json()['nextPageCursor'] == 'None'):
            Done = True
    return PastNames

def IsOnline(UserID):
    """
    Returns whether a user is online
    """
    response = requests.get(Utils.UserAPI + str(UserID))
    try:
        return response.json()['IsOnline']
    except:
        return 'User not found'

def Isbanned(UserID):
    """
    Returns if a user account is currently banned
    """
    response = requests.get(Utils.UserAPIV1 + str(UserID))
    try:
        return response.json()['isBanned']
    except:
        return 'User not found'

def GetDescription(UserID):
    """
    Returns the description of a given user
    """
    response = requests.get(Utils.UserAPIV1 + str(UserID))
    try:
        return response.json()['description']
    except:
        return 'User not found'

def GetAge(UserID):
    """
    Returns the user's age in days
    """
    response = requests.get(Utils.UserAPIV1 + str(UserID))
    try:
        CreationDate = response.json()['created']
        CreationDate = CreationDate.split('T')
        CreationDate = CreationDate[0].split('-')
        CreationDate = datetime.date(int(CreationDate[0]), int(CreationDate[1]), int(CreationDate[2]))
        Days = ((datetime.date.today()) - (CreationDate))
        Days = str(Days).split(' ')
        return Days[0]
    except:
        return (Utils.UserAPIV1 + str(UserID))

def CreationDate(UserID, Style = 0):
    """
    Returns the date a user was created

    StandardFormat: dd/mm/yyyy
    Americans: mm/dd/yyyy

    If you wish you to use the American format set the 'Style' Vairable to 1

    """
    response = requests.get(Utils.UserAPIV1 + str(UserID))
    try:
        CreationDate = response.json()['created']
        CreationDate = CreationDate.split('T')
        CreationDate = CreationDate[0].split('-')
        if(Style == 0):
            return (str(CreationDate[2]) + '/' + str(CreationDate[1]) + '/' + str(CreationDate[0])) #DD/MM/YYYY -- The Correct Format
        else:
            return (str(CreationDate[1]) + '/' + str(CreationDate[2]) + '/' + str(CreationDate[0])) #MM/DD/YYYY -- Those Colonists Format
    except:
        return response.json()['errors'][0]['message']

def GetRAP(UserID):
    """
    Returns the total offical roblox RAP value for a user

    Please be aware this function can take some time to run depending on internet speed and how many limiteds a user owns
    """
    ErroredRAP = 0
    TotalValue = 0
    Cursor = ""
    Done = False
    while(Done == False):
        try:
            response = requests.get(Utils.Inventory1URL + f"/{UserID}/assets/collectibles?sortOrder=Asc&limit=100&cursor={Cursor}")
            Items = response.json()
            if((response.json()['nextPageCursor'] == "null") or response.json()['nextPageCursor'] == None):
                Done = True
            else:
                Done = False
                Cursor = response.json()['nextPageCursor']
            for Item in Items["data"]:
                try:
                    RAP = int((Item['recentAveragePrice']))
                    TotalValue = TotalValue + RAP
                except:
                    TotalValue = TotalValue
            if(response.json()['nextPageCursor'] == 'None'):
                Done = True
            
        except Exception as ex:
            Done = True
    return(TotalValue)

def GetLimiteds(UserID):
    """
    Returns the total list of a users limiteds

    Please be aware this function can take some time to run depending on internet speed and how many limiteds a user owns
    """
    Limiteds = []
    IDs = []
    Cursor = ""
    Done = False
    while(Done == False):
        try:
            response = requests.get(Utils.Inventory1URL + f"/{UserID}/assets/collectibles?sortOrder=Asc&limit=100&cursor={Cursor}")
            Items = response.json()
            if((response.json()['nextPageCursor'] == "null") or response.json()['nextPageCursor'] == None):
                Done = True
            else:
                Done = False
                Cursor = response.json()['nextPageCursor']
            for Item in Items["data"]:
                try:
                    Limited = Item['name']
                    ID = Item['assetId']
                    Limiteds.append(Limited)
                    IDs.append(ID)
                except:
                    Limiteds = Limiteds
                    IDs = IDs
            if(response.json()['nextPageCursor'] == 'None'):
                Done = True
            
        except Exception as ex:
            Done = True
    return(Limiteds,IDs)

def GetBust(UserID,Width = 420, Height = 420):
    """
    Returns the link to a bust image of a user

    Width and Height can be customised
    """
    response = requests.get(f"https://www.roblox.com/bust-thumbnail/image?userId={UserID}&width={Width}&height={Height}&format=png")
    return response.url

def GetHeadshot(UserID,Width = 420, Height = 420):
    """
    Returns the link to a headshot image of a user

    Width and Height can be customised
    """
    response = requests.get(f"https://www.roblox.com/headshot-thumbnail/image?userId={UserID}&width={Width}&height={Height}&format=png")
    return response.url

def GetStatus(UserID):
    """
    Returns the current status of a user
    """
    response = requests.get(Utils.UserAPIV1 + f"{str(UserID)}/status")
    return response.json()['status']

def DoesNameExist(Username):
    response = requests.get(Utils.UserAPIV1 + 'get-by-username?username=' + str(Username))
    try:
        if('errorMessage' in response.text):
            return ('Availible')
        else:
            if(response.json()['Username'].lower() == Username.lower()):
                return('Unavailible')
            elif (response.json()['Username'].lower() != Username.lower()):
                return('Availible')
    except:
        return response.json()


import requests,json
from robloxpy import Utils as Utils

def GetAll(UserID):
    """
    Returns a full list of a users friends
    """
    FullList = []
    response = requests.get(Utils.UserAPI + str(UserID) + "/friends")
    response = requests.get(Utils.FriendsAPI + f'{UserID}/friends')
    Friendslist = json.loads(response.text)
    for friend in Friendslist['data']:
        FullList.append(friend['displayName'])
    return FullList

def GetCount(UserID):
    """
    Returns the total number of friends a user has
    """
    response = requests.get(Utils.FriendsAPI + f"{UserID}/friends/count")
    try:
        return response.json()['count']
    except:
        return response.json()['errors'][0]['message']

def GetOnline(UserID):
    """
    Returns a full list of a users friends that are currently online
    """
    FullList = []
    response = requests.get(Utils.FriendsAPI + f"{UserID}/friends/")
    try:
        Friendslist = json.loads(response.text)
        for friend in Friendslist['data']:
            if(friend['isOnline'] == True):
                FullList.append(friend['displayName'])
        return FullList
    except:
        return response.json()['errors'][0]['message']

def GetOffline(UserID):
    """
    Returns a full list of a users friends that are currently offline
    """
    FullList = []
    response = requests.get(Utils.FriendsAPI + f"{UserID}/friends/")
    try:
        Friendslist = json.loads(response.text)
        for friend in Friendslist['data']:
            if(friend['isOnline'] == False):
                FullList.append(friend['displayName'])
        return FullList
    except:
        return response.json()['errors'][0]['message']

def GetFollowerCount(UserID):
    """
    Returns a count of how many users a person is following
    """
    response = requests.get(Utils.FriendsAPI + f"{UserID}/followers/count")
    try:
        return response.json()['count']
    except:
        return response.json()['errors'][0]['message']

def GetFollowers(UserID,Amount):
    """
    Returns a full list of people that the user follows

    [Username],[ID]
    """
    response = requests.get(Utils.FriendsAPI + str(UserID) + '/followers?sortOrder=Asc&limit=100')
    try:
        CurrentAmount = 0
        NameList = []
        IDList = []
        Cursor = 'None'
        #print(str(Utils.FriendsAPI + str(UserID) + '/followers?sortOrder=Asc&limit=100'))
        while Cursor != 'null':
            if(Cursor is None):
                Cursor = 'null'
            else:
                try:
                    Cursor = response.json()['nextPageCursor']
                except:
                    Cursor = 'null'
            for Follower in response.json()['data']:
                if(CurrentAmount < Amount):
                    NameList.append(Follower['name'])
                    IDList.append(Follower['id'])
                    CurrentAmount = CurrentAmount + 1
                else:
                    return NameList, IDList
            if(Cursor is None):
                Cursor = 'null'
            else:
                try:
                    response = requests.get(Utils.FriendsAPI + 'users/' + str(UserID) + '/followers?sortOrder=Asc&limit=100&cursor=' + str(Cursor))
                except:
                    Cursor = 'null'
        return NameList, IDList
    except:
        return NameList, IDList

def GetFollowingCount(UserID):
    """
    Returns a count of how many users a person is following
    """
    response = requests.get(Utils.FriendsAPI + f"{UserID}/followings/count")
    try:
        return response.json()['count']
    except:
        return response.json()['errors'][0]['message']

def GetFollowing(UserID,Amount):
    """
    Returns a full list of people that follow the user

    [Username],[ID]
    """
    response = requests.get(Utils.FriendsAPI + str(UserID) + '/followings?sortOrder=Asc&limit=100')
    try:
        CurrentAmount = 0
        NameList = []
        IDList = []
        Cursor = 'None'
        while Cursor != 'null':
            if(Cursor is None):
                Cursor = 'null'
            else:
                try:
                    Cursor = response.json()['nextPageCursor']
                except:
                    Cursor = 'null'
            for Follower in response.json()['data']:
                if(CurrentAmount < Amount):
                    NameList.append(Follower['name'])
                    IDList.append(Follower['id'])
                    CurrentAmount = CurrentAmount + 1
                else:
                    return NameList, IDList
            if(Cursor is None):
                Cursor = 'null'
            else:
                try:
                    response = requests.get(Utils.FriendsAPI + str(UserID) + '/followers?sortOrder=Asc&limit=100&cursor=' + str(Cursor))
                except:
                    Cursor = 'null'
        return NameList, IDList
    except:
        return NameList, IDList


import robloxpy.Utils as Utils
import requests


def GetUniverseData(UniverseID):
    response = requests.get(f"{Utils.GamesAPI}games?universeIds={str(UniverseID)}")
    try:
        return response.json()['data'][0]
    except:
        return 'Universe Not Found'

def GetUniverseVotes(UniverseID):
    response = requests.get(f"{Utils.GamesAPI}games/votes?universeIds={str(UniverseID)}")
    try:
        return response.json()['data'][0]
    except:
        return 'Universe Not Found'

def GetUniverseFavourites(UniverseID):
    response = requests.get(f"{Utils.GamesAPI}games/{str(UniverseID)}/favorites/count")
    try:
        return response.json()['favoritesCount']
    except:
        return 'Universe Not Found'

def GetCurrentUniversePlayers(UniverseID):
    GameData = GetUniverseData(str(UniverseID))
    try:
        return GameData['playing']
    except:
        return 'Universe Not Found'

def GetUniverseVisits(UniverseID):
    GameData = GetUniverseData(str(UniverseID))
    try:
        return GameData['visits']
    except:
        return 'Universe Not Found'

def GetUniverseLikes(UniverseID):
    GetVotes = GetUniverseVotes(str(UniverseID))
    try:
        return GetVotes['upVotes']
    except:
        return 'Universe Not Found'

def GetUniverseDislikes(UniverseID):
    GetVotes = GetUniverseVotes(str(UniverseID))
    try:
        return GetVotes['downVotes']
    except:
        return 'Universe Not Found'


HasRequests = False
HasJson = False

#Check Requirements Exist
try:
    import requests
    HasRequests = True
except:
    HasRequests = False

try:
    import json
    HasJson = True
except:
    HasJson = False

if(HasRequests == False and HasJson == False):
    print("Missing requests and json")
elif(HasRequests == False and HasJson == True):
    print("Please run the following command in cmd/terminal")
    print("pip install requests")
elif(HasJson == False):
    print("Python installation error | json missing")

#import robloxpy.Utils as Utils
#import robloxpy.Game as Game
#import robloxpy.User as User
#import robloxpy.Market as Market    
#import robloxpy.Group as Group


import requests,json
from robloxpy import Utils as Utils

def CanManageAsset(UserID,AssetID):
    """
    Returns whether a user can manage an asset
    """
    response = requests.get(Utils.APIURL + f"/users/{UserID}/canmanage/{AssetID}")
    try:
        return response.json()['CanManage']
    except:
        return response.json()['ErrorMessage']

def GetLimitedPriceData(LimitedID):
    """
    Returns price data of a limited
    """
    response = requests.get(Utils.EconomyURL + '/assets/' + str(LimitedID) + '/resale-data')
    return response.json()['priceDataPoints']

def GetLimitedRemaining(LimitedID):
    """
    Returns the amount of limiteds that are availible for sale
    """
    response = requests.get(Utils.EconomyURL + '/assets/' + str(LimitedID) + '/resale-data')
    return response.json()['numberRemaining']

def GetLimitedTotal(LimitedID):
    """
    Returns how many of a limited there are
    """
    response = requests.get(Utils.EconomyURL + '/assets/' + str(LimitedID) + '/resale-data')
    return response.json()['assetStock']

def GetLimitedSales(LimitedID):
    """
    Returns how many of a limited have been sold
    """
    response = requests.get(Utils.EconomyURL + '/assets/' + str(LimitedID) + '/resale-data')
    return response.json()['sales']

def GetLimitedRAP(LimitedID):
    """
    Returns the RAP of a limited
    """
    response = requests.get(Utils.EconomyURL + '/assets/' + str(LimitedID) + '/resale-data')
    return response.json()['recentAveragePrice']

def GetLimitedSalePrice(LimitedID):
    """
    Returns the sale price of a limited
    """
    response = requests.get(Utils.EconomyURL + '/assets/' + str(LimitedID) + '/resale-data')
    return response.json()['originalPrice']

def GetLimitedChangePercentage(LimitedID):
    """
    Returns the percentage change of the price of a limited
    """
    Change = round(((float(GetLimitedRAP(LimitedID))-GetLimitedSalePrice(LimitedID))/GetLimitedSalePrice(LimitedID))*100,3)
    return str(Change) + '%'

def GetAssetImage(AssetID,Width = 420,Height = 420):
    """
    Returns a url to an image of the limited

    Limited image size can be changed
    """
    response = requests.get(f"https://www.roblox.com/asset-thumbnail/image?assetId={AssetID}&width={Width}&height={Height}&format=png")
    return response.url


import robloxpy.Utils as Utils
import robloxpy.User.Internal as Internal

def Claim(GroupID):
    try:
        response = Internal.CurrentCookie.post(f"{Utils.GroupAPIV1}{GroupID}/claim-ownership")
        return 'Sent'
    except:
        return response.json()

def Join(GroupID):
    try:
        response = Internal.CurrentCookie.post(f"{Utils.GroupAPIV1}{GroupID}/users")
        if('captcha test' in response.content.decode()): #Credit https://github.com/PurityWasHere
            return 'Captcha'
        else:
            return 'Join request sent'
    except:
        return response.json()

def Leave(GroupID):
    try:
        response = Internal.CurrentCookie.delete(f"{Utils.GroupAPIV1}{GroupID}/users/{Internal.UserID}")
        return 'Sent'
    except:
        return response.json()

def GetFunds(GroupID):
    try:
        response = Internal.CurrentCookie.post(f"{Utils.EconomyURL}/groups/{GroupID}/currency/")
        return response.json()['robux']
    except:
        return response.json()

def Payout(GroupID,targetUserID,RobuxAmount):
    try:
        data = {
            "PayoutType": "FixedAmount",
            "Recipients": [
                {
                    "recipientId": str(targetUserID),
                    "recipientType": "User",
                    "amount": str(RobuxAmount)
                }
            ]
        }
        response = Internal.CurrentCookie.post(f"{Utils.GroupAPIV1}{GroupID}/payouts", json=data)
        if(response.status_code == 200):
            return 'Sent'
        else:
            return response.json()
    except Exception as e:
        return e

def PercentagePayout(GroupID,targetUserID,Percentage):
    try:
        data = {
            "PayoutType": "Percentage",
            "Recipients": [
                {
                    "recipientId": str(targetUserID),
                    "recipientType": "User",
                    "amount": str(Percentage)
                }
            ]
        }
        response = Internal.CurrentCookie.post(f"{Utils.GroupAPIV1}{GroupID}/payouts", json=data)
        if(response.status_code == 200):
            return 'Sent'
        else:
            return response.json()
    except Exception as e:
        return e

def SendWallPost(GroupID,PostText):
    try:
        response = Internal.CurrentCookie.post(f"{Utils.GroupAPIV1}{GroupID}/wall/posts", data={'body': PostText})
        return 'Sent'
    except:
        return response.json()

def SendGroupShout(GroupID,ShoutText):
    try:
        response = Internal.CurrentCookie.patch(f"{Utils.GroupAPIV1}{GroupID}/status", data={'message': ShoutText})
        return 'Sent'
    except:
        return response.json()

def ChangeDescription(GroupID, DescriptionText):
    try:
        response = Internal.CurrentCookie.patch(f"{Utils.GroupAPIV1}{GroupID}/description", data={'description': DescriptionText})
        return 'Sent'
    except:
        return response.json()

def ChangeRank(GroupID,targetUserID,RoleID):
    try:
        response = Internal.CurrentCookie.patch(f"{Utils.GroupAPIV1}{GroupID}/users/{targetUserID}", data={'roleId': RoleID})
        return 'Sent'
    except:
        return response.json()








import robloxpy.Utils as Utils
import robloxpy.User.Internal as Internal

def SendFriendRequest(targetUserID):
    """
    Send a friend request to the target
    """
    try:
        response = Internal.CurrentCookie.post(f"{Utils.FriendsAPI}users/{str(targetUserID)}/request-friendship", data={'targetUserID': targetUserID})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']
    except Exception as e:
        return response.json()

def Unfriend(targetUserID):
    """
    Unfriend the target user
    """
    try:
        response = Internal.CurrentCookie.post(f"{Utils.FriendsAPI}users/{str(targetUserID)}/unfriend", data={'targetUserID': targetUserID})
        try:
            return response.json()['success']
        except:
            return response.json()['errors']
    except Exception as e:
        return response.json()

def TotalFriends():
    """
    Returns the total amount of friends
    """
    try:
        response = Internal.CurrentCookie.get(f"{Utils.FriendsAPI}my/friends/count")
        return response.json()['count']
    except:
        return response.json()